-- Overwatch, Hearthstone and World of Warcraft Copyright © 2016 by Blizzard Entertainment

local addonName, S = ...
local L = S.L -- localized emotes from other players

-- only English supported for now
local locales = {
	enUS = true,
	--deDE = true,
	--frFR = true,
	--ruRU = true,
	--zhCN = true,
	--zhTW = true,
	--koKR = true,
}

local clientLocale = GetLocale()
local addonLocale = locales[clientLocale] and clientLocale or "enUS" -- default back to enUS

local sound = {
	hello = {
		-- missing D.Va English hello
		"D.Va\\kor_1.mp3", -- native language
		"Mei\\hi.ogg",
		"Mei\\chinese 13.mp3", -- native language
		"Mercy\\hello.mp3",
		"Tracer\\hiya.mp3",
		"Tracer\\cheers.mp3",
		"McCree\\hello.mp3",
		"McCree\\heythere.mp3",
		"Reinhardt\\hello.mp3",
		"Jaina\\VO_HERO_08_Greetings_56.ogg", -- hearthstone
	},
	laugh = {
		"D.Va\\laugh.mp3",
		"D.Va\\lol.mp3",
		"Mei\\laugh.mp3",
	},
	thanks = {
		"D.Va\\thanks.mp3",
		"Mei\\thank you.mp3",
		"Mercy\\thank you.mp3",
		"Tracer\\thanks loves.mp3",
	},
	sorry = {
		"Mei\\sorry.mp3",
		"Mei\\sorry sorry sorry.ogg",
		"Jaina\\VO_HERO_08_Sorry_61.ogg", -- hearthstone
	},
	amazing = {
		"Mei\\thats amazing.mp3",
		"Mei\\wow.mp3",
		"Tracer\\great scott.mp3",
	},
	brb = {
		"D.Va\\brb.mp3",
		"Tracer\\brb.mp3",
	},
	afk = {
		"D.Va\\afks.mp3",
	},
	spawn = {
		"D.Va\\game_on_2_.mp3",
		"D.Va\\im too young.mp3",
		"D.Va\\i play to win.mp3",
		"D.Va\\im back in the game.mp3",
		"Mei\\our world is worth fighting for.mp3",
		"Mei\\our planet is.mp3",
		"Mercy\\back in the fight.mp3",
		"Tracer\\cheers love.mp3",
		"Tracer\\keep calm and.mp3",
		"Tracer\\the world could.mp3",
	},
	mass_rez = {
		"Mercy\\heroes never die.mp3",
	},
	countdown = {
		"D.Va\\5 4 3 2 1.mp3",
		"Mei\\5 4 3 2 1.mp3",
		"Mei\\5 4 3 2 1 chinese.mp3", -- native language
		"Mercy\\5 4 3 2 1.mp3",
		"Tracer\\5 4 3 2 1.mp3",
		"Widowmaker\\5 4 3 2 1.mp3",
		"McCree\\5 4 3 2 1.mp3",
		"Reinhardt\\5 4 3 2 1.mp3",
		"Soldier76\\5 4 3 2 1.mp3",
	},
	blink = {
		"Tracer\\blink1.ogg",
		"Tracer\\blink2.ogg",
		"Tracer\\blink3.ogg",
		"Tracer\\blink4.ogg",
	},
	levelup = {
		"D.Va\\level_up.mp3",
	},
	gg = {
		"D.Va\\GG.mp3",
	},
	glhf = {
		"D.Va\\glhf.mp3",
	},
	ready = {
		"Tracer\\ready.mp3",
		"Mercy\\im ready.mp3",
	},
	easy = {
		"D.Va\\is this ez mode.mp3",
		"D.Va\\are you even trying.mp3",
		"McCree\\easy.mp3",
	},
	--[[
	dva = {
		"D.Va\\alright game face on.mp3",
		"D.Va\\dont_give_up.mp3",
		"D.Va\\epic.mp3",
		"D.Va\\multi kill.mp3",
		"D.Va\\mvp.mp3",
		"D.Va\\no hacks required.mp3",
		"D.Va\\no ones beating my high scores.mp3",
		"D.Va\\top of the scoreboard.mp3",
	},
	]]
}

-- only plays a sound when you do an /emote
-- have to do locale specific strfind in order to support emotes from other players
local emote = {
	HELLO = "hello",
	GREET = "hello",
	WAVE = "hello",
	LAUGH = "laugh",
	ROFL = "laugh",
	THANK = "thanks",
	APOLOGIZE = "sorry",
	AMAZE = "amazing",
	APPLAUD = "amazing",
	BRB = "brb",
	READY = "ready",
}

-- emotes that dont trigger any sound
local nodelay = {
	GREET = true,
	WAVE = true,
	BRB = true,
	AMAZE = true,
	APPLAUD = true,
	--APOLOGIZE = true, -- voice added in Legion
	READY = true,
}

local cd = {}

local function PlayRandomSound(k)
	if time() > (cd.sound or 0) then
		cd.sound = time() + 1 -- throttle
		
		local t = sound[k]
		PlaySoundFile("Interface\\AddOns\\OverwatchSounds\\Sounds\\"..addonLocale.."\\"..t[random(#t)], "MASTER")
	end
end

local e = {} -- events
local f = CreateFrame("Frame")

-- resurrections
function e:COMBAT_LOG_EVENT_UNFILTERED(_, subevent, _, sourceGUID, sourceName, _, _, destGUID, destName, _, _, spellID, spellName)
	if subevent == "SPELL_RESURRECT" then
		-- lazy way to check if it might be a battlerez
		if UnitAffectingCombat("player") then
			PlayRandomSound("mass_rez")
		end
	end
end

function e:UNIT_SPELLCAST_SUCCEEDED(unit, spellName, _, _, spellID)
	if unit == "player" and spellID == 1953 then
		-- delay because no idea what the Blink sound files are
		C_Timer.After(.5, function()
			PlayRandomSound("blink")
		end)
	end
end

-- alive after being a ghost
function e:PLAYER_UNGHOST()
	C_Timer.After(2, function()
		PlayRandomSound("spawn")
	end)
end

local MARKED_AFK_MESSAGE = gsub(MARKED_AFK_MESSAGE, "%%s", ".+")

-- AFK
function e:CHAT_MSG_SYSTEM(msg)
	if msg == MARKED_AFK or strfind(msg, MARKED_AFK_MESSAGE) then
		PlayRandomSound("afk")
	end
end

local function CHAT_MSG(msg)
	if strfind(msg, "lol") and strlen(msg) <= 5 then
		PlayRandomSound("laugh")
	elseif (strfind(msg, "hello") or strfind(msg, "hey")) and strlen(msg) <= 6 then
		PlayRandomSound("hello")
	elseif (strfind(msg, "thank") or strfind(msg, "thx")) and strlen(msg) <= 7 then
		PlayRandomSound("thanks")
	elseif strfind(msg, "afk") and strlen(msg) <= 10 then
		PlayRandomSound("afk")
	elseif strfind(msg, "gg") and strlen(msg) <= 4 then
		PlayRandomSound("gg")
	elseif strfind(msg, "brb") and strlen(msg) <= 4 then
		PlayRandomSound("brb")
	elseif strfind(msg, "glhf") and strlen(msg) <= 5 then
		PlayRandomSound("glhf")
	elseif (strfind(msg, "ez") or strfind(msg, "easy")) and strlen(msg) <= 5 then
		PlayRandomSound("easy")
	end
end

function e:CHAT_MSG_SAY(msg)
	CHAT_MSG(msg)
end

function e:CHAT_MSG_YELL(msg)
	CHAT_MSG(msg)
end

function e:CHAT_MSG_WHISPER(msg)
	CHAT_MSG(msg)
end

function e:CHAT_MSG_CHANNEL(msg)
	CHAT_MSG(msg)
end

function e:CHAT_MSG_INSTANCE_CHAT(msg)
	CHAT_MSG(msg)
end

-- emotes by self
hooksecurefunc("DoEmote", function(token)
	if emote[token] then
		if nodelay[token] then
			PlayRandomSound(emote[token])
		else
			-- delay Overwatch sounds a bit
			-- since normal sounds still play and takes too much work removing them
			C_Timer.After(1, function()
				PlayRandomSound(emote[token])
			end)
		end
	end
end)

local playerGUID

C_Timer.After(.1, function() -- not immediately available at login
	playerGUID = UnitGUID("player")
end)

-- emotes by other players
function e:CHAT_MSG_TEXT_EMOTE(msg, ...)
	local guid = select(11, ...)
	
	for k, v in pairs(L) do
		if strfind(msg, v) then
			-- should never run for the player with the enUS locale
			-- but just to be sure this is only from other players
			if guid ~= playerGUID then
				PlayRandomSound(emote[k])
			end
			return
		end
	end
end

function e:PLAYER_LEVEL_UP()
	C_Timer.After(1, function()
		PlayRandomSound("levelup")
	end)
end

local timer

-- battleground preparation countdown
hooksecurefunc("StartTimer_SwitchToLargeDisplay", function()
	-- DBM load order might be after this addon, check in the posthook
	if not timer and not IsAddOnLoaded("DBM-Core") then
		PlayRandomSound("countdown")
		timer = true
		-- StartTimer_SwitchToLargeDisplay runs every second
		C_Timer.After(6, function()
			timer = false
		end)
	end
end)

local LSM = LibStub("LibSharedMedia-3.0")

-- add to SharedMedia
for _, v in pairs(sound) do
	for _, v2 in pairs(v) do
		LSM:Register(LSM.MediaType.SOUND, "OW "..v2:gsub("\\", " "), "Interface\\AddOns\\OverwatchSounds\\Sounds\\"..addonLocale.."\\"..v2)
	end
end

-- initialize
for k in pairs(e) do
	f:RegisterEvent(k)
end

f:SetScript("OnEvent", function(self, event, ...)
	e[event](self, ...)
end)
